 // uncomment the following to debug the HTML, CSS and JS in the browser 
 //
 //  addStartActionToTop('New', 'Start new file');
 //  addStartActionToBottom('MATLAB', 'Start with MATLAB');
 //  addLearnAction('Need a job?', 'Send us your application!');
 //  addRecentFile('foo_blahblah_super_long_file_name_looooong.tpt', 'C:\\my\\path\\foo_blahblah_super_long_file_name_____.tpt', '6 days ago');
 //  addRecentFile('bar.tpt', 'C:\\my\\path\\bar.tpt', '2 weeks ago');
 //  addExample('01 Getting Started', '10 First Example', 'A very first example');
 //  addExample('01 Getting Started', 'Second Example', 'A very second example');
 //  addExample('02 Test Modeling', 'Step Types', 'Explore the different step types and moooooooree. Super loooooong descriptin!!1!');
 //  setTheme('default');
 //  document.getElementById('theme-switch').addEventListener('change', (evt) => {
 //      setTheme(evt.target.checked ? 'dark' : 'default');
 //  });

 /**
  * 
  * @param {string} theme 
  */
 function setTheme(theme) {
     if (theme === 'dark') {
         document.body.setAttribute('data-theme', theme);
     } else {
         document.body.setAttribute('data-theme', 'default');
     }
 }

 /**
  * 
  * @param {string} title 
  * @param {string} description 
  * @returns button of action
  */
 function addStartActionToTop(title, description) {
     let actionElem = _createButton(title, description);
     document.getElementById('start-top-section').append(actionElem);
     return actionElem.firstChild.firstChild;
 }

 /**
  * 
  * @param {string} title 
  * @param {string} description 
  * @returns button of action
  */
 function addStartActionToBottom(title, description) {
     document.getElementById('start-bottom-header').removeAttribute('hidden');
     let actionElem = _createButton(title, description);
     document.getElementById('start-bottom-section').append(actionElem);
     return actionElem.firstChild.firstChild;
 }

 /**
  * 
  * @param {string} title 
  * @param {string} description 
  * @returns button of action
  */
 function addConfigureAction(title, description) {
     let actionElem = _createButton(title, description);
     document.getElementById('configure-section').append(actionElem);
     return actionElem.firstChild.firstChild;
 }

 /**
  * 
  * @param {string} title 
  * @param {string} description 
  * @returns button of action
  */
 function addLearnAction(title, description) {
     let actionElem = _createButton(title, description);
     document.getElementById('learn-section').append(actionElem);
     return actionElem.firstChild.firstChild;
 }

 /**
  * 
  */
 function addNoRecentFileHint() {
     let recentFilesList = document.getElementById('recent-files-list');
     if (recentFilesList !== undefined && recentFilesList !== null) {
         recentFilesList.remove();
     }
     let hint = document.createElement('h6');
     hint.id = 'no-recent-files-hint';
     hint.classList.add('text-center');
     hint.innerText = 'No Recent Files';
     document.getElementById('recent-files-section').append(hint);
 }

 /**
  * 
  * @param {string} fileName 
  * @param {string} filePath 
  * @param {string} openedAt 
  * @returns button of file
  */
 function addRecentFile(fileName, filePath, openedAt) {
     let noRecentFileHint = document.getElementById('no-recent-file-hint');
     if (noRecentFileHint !== undefined && noRecentFileHint !== null) {
         noRecentFileHint.remove();
     }
     let recentFilesList = document.getElementById('recent-files-list');
     if (recentFilesList === undefined || recentFilesList === null) {
         recentFilesList = document.createElement('div');
         recentFilesList.id = 'recent-files-list';
         recentFilesList.classList.add('list-group', 'list-group-flush');
         document.getElementById('recent-files-section');
     }
     let recentFileButton = _createRecentFileButton(fileName, filePath, openedAt);
     recentFilesList.append(recentFileButton);
     return recentFileButton;
 }

 function addNoExampleHint() {
     let examplesAcc = document.getElementById('examples-accordion');
     if (examplesAcc.hasChildNodes()) {
         examplesAcc.innerHTML = '';
     }
     let hint = document.createElement('h6');
     hint.id = 'no-examples-hint';
     hint.classList.add('text-center');
     hint.innerText = 'No Examples Available';
     examplesAcc.append(hint);
 }

 /**
  * 
  * @param {string} groupName 
  * @param {string} fileName 
  * @param {string} tooltip 
  * @returns example button
  */
 function addExample(groupName, fileName, tooltip) {
     let groupNameWoWs = _makeStringIdUsable(groupName);
     let groupBody = document.getElementById(groupNameWoWs + '-body');
     if (groupBody === undefined || groupBody === null) {
         let groupElem = _createExampleGroupAccordionItem(groupName);
         document.getElementById('examples-accordion').append(groupElem);
         groupBody = document.getElementById(groupNameWoWs + '-body');
     }
     let exampleButton = _createExampleButton(fileName, tooltip);
     groupBody.append(exampleButton);
     return exampleButton;
 }

 /**
  * creates a button as follows:
  * 
  * <div class="row">
  *  <div class="col">
  *   <button type="button" class="btn btn-flat-laf text-start w-100 lh-1">
  *    <h6>title</h6>
  *    <small class="mb-0 text-muted">description</small>
  *   </button>
  *  </div>
  * </div>
  * 
  * @param {string} title
  * @param {string} description
  * @return 
  */
 function _createButton(title, description) {
     let row = document.createElement('div');
     row.classList.add('row');
     let col = document.createElement('div');
     row.classList.add('col');
     let button = document.createElement('button');
     button.classList.add('btn', 'btn-flat-laf', 'text-start', 'w-100', 'lh-1');
     button.setAttribute('type', 'button');
     let h5 = document.createElement('h6');
     h5.innerText = title;
     let small = document.createElement('small');
     small.classList.add('mb-0', 'text-muted');
     small.innerText = description;

     button.append(h5, small);
     col.append(button);
     row.append(col);
     return row;
 }

 /**
  * creates a button as follows:
  * 
  * <button type="button" class="list-group-item list-group-item-action">
  *  <div class="d-flex w-100 justify-content-between">
  *   <h6 class="mb-2 pe-4 text-break">fileName</h6>
  *   <small class="text-muted text-end">openedAt</small>
  *  </div>
  *  <small class="text-muted text-break">filePath</small>
  * </button>
  *
  * @param {string} fileName
  * @param {string} filePath
  * @param {string} openedAt
  * @return 
  */
 function _createRecentFileButton(fileName, filePath, openedAt) {
     let button = document.createElement('button');
     button.classList.add('list-group-item', 'list-group-item-action');
     button.setAttribute('type', 'button');
     let div = document.createElement('div');
     div.classList.add('d-flex', 'w-100', 'justify-content-between');
     let fileNameHeader = document.createElement('h6');
     fileNameHeader.classList.add('mb-2', 'pe-4', 'text-break');
     fileNameHeader.innerText = fileName;
     let openedAtSmall = document.createElement('small');
     openedAtSmall.classList.add('text-muted', 'text-end');
     openedAtSmall.innerText = openedAt;
     let filePathSmall = document.createElement('small');
     filePathSmall.classList.add('text-muted', 'text-break');
     filePathSmall.innerText = filePath;

     div.append(fileNameHeader, openedAtSmall);
     button.append(div, filePathSmall);
     return button;
 }

 /**
  * creates an accordion item as follows:
  * 
  * <div class="accordion-item border-top border-bottom">
  *  <h2 class="accordion-header">
  *   <button class="accordion-button collapsed h6 mb-0" type="button" data-bs-toggle="collapse" data-bs-target="#group-name-collapse">
  *    groupName
  *   </button>
  *  </h2>
  *  <div id="group-name-collapse" class="accordion-collapse collapse" data-bs-parent="#examples-accordion">
  *   <div class="accordion-body p-0" id="group-name-body">
  *     list of example buttons
  *   </div>
  *  </div>
  * </div>
  *
  * @param {string} groupName
  * @return 
  */
 function _createExampleGroupAccordionItem(groupName) {
     let groupNameWoSpace = _makeStringIdUsable(groupName);
     let accItemDiv = document.createElement('div');
     accItemDiv.classList.add('accordion-item', 'border-bottom');
     let accHeader = document.createElement('h2');
     //  accHeader.id = groupNameWoSpace + '-header';
     accHeader.classList.add('accordion-header');
     let collapseButton = document.createElement('button');
     collapseButton.classList.add('accordion-button', 'collapsed', 'h6', 'mb-0');
     collapseButton.setAttribute('type', 'button');
     collapseButton.setAttribute('data-bs-toggle', 'collapse');
     collapseButton.setAttribute('data-bs-target', '#' + groupNameWoSpace + '-collapse');
     collapseButton.textContent = ripOffIndex(groupName);
     let accCollapse = document.createElement('div');
     accCollapse.id = groupNameWoSpace + '-collapse';
     accCollapse.classList.add('accordion-collapse', 'collapse');
     accCollapse.setAttribute('data-bs-parent', '#examples-accordion');
     let accBody = document.createElement('div');
     accBody.id = groupNameWoSpace + '-body';
     accBody.classList.add('accordion-body', 'p-0');

     accHeader.append(collapseButton);
     accCollapse.append(accBody);
     accItemDiv.append(collapseButton, accCollapse);
     return accItemDiv;
 }

 /**
  * creates a button as follows:
  * 
  * <button type="button" class="list-group-item list-group-item-action lh-1">
  *  <h6 class="mb-2">fileName</h6>
  *  <small class="text-muted">tooltip</small>
  * </button>
  *
  * @param {string} fileName 
  * @param {string} tooltip 
  * @return
  */
 function _createExampleButton(fileName, tooltip) {
     let button = document.createElement('button');
     button.classList.add('list-group-item', 'list-group-item-action', 'lh-1');
     button.setAttribute('type', 'button');
     let fileNameHeader = document.createElement('h6');
     fileNameHeader.classList.add('mb-2');
     fileNameHeader.innerText = ripOffIndex(fileName);
     let tooltipSmall = document.createElement('small');
     tooltipSmall.classList.add('text-muted');
     tooltipSmall.innerText = tooltip;

     button.append(fileNameHeader, tooltipSmall);
     return button;
 }

 /**
  * rips off the index of a string
  * 
  * `01 Getting Started` -> `Getting Started`
  * 
  * @param {string} input 
  * @returns 
  */
 function ripOffIndex(input) {
     const numRegex = /^\d*/g;
     const output = input.replace(numRegex, '').trim();
     return output;
 }

 /**
  * removes all numbers and replaces all whitespace with '-'
  * 
  * @param {string} input 
  * @returns 
  */
 function _makeStringIdUsable(input) {
     const numRegex = /\d/g;
     input = input.replace(numRegex, '');
     input = input.trim();
     const wsRegex = /\s/g;
     return input.replace(wsRegex, '-');
 }