/*********************************************************************
 *              Copyright (c) 2021 PikeTec GmbH
 *               CONFIDENTIAL AND PROPRIETARY.
 *                   ALL RIGHTS RESERVED
 *              - PROPRIETARY OF PikeTec GmbH -
 **********************************************************************/
/* clang-format off */

#ifndef __TPT_FUSION_OBSOLETE_H_
#define __TPT_FUSION_OBSOLETE_H_

#include "tpt_fusion_base.h"
#include "tpt_fusion.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * All functions declared in this header file should be treated as "deprecated". There are
 * always "official" counterpart functions declared in the tpt_fusion.h header which are
 * the recommendend functions to be used by customers.
 */

/* ============================= OBSOLETE/DEPRECATED FUSION FUNCTIONS =============================== */

/**
 * Creates the counterpart of a node's input measurement for the fusion session, 
 * in order to be able to read that input during a fusion session.
 *
 * @param handle
 *          The handle of the calling node.
 * @param name
 *          The name of the input measurement to connect to the fusion.
 * @return A handle to the output signal for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareScalar() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_signal_handle) tpt_fusion_declareInputAsDouble(tpt_fusion_node_handle handle, 
                                                         const char *name);


/**
 * Creates the counterpart of a node's output measurement for the fusion session,
 * in order to be able to read that output during a fusion session.
 *
 * @param handle
 *          The handle of the calling node.
 * @param name
 *          The name of the output measurement to connect to the fusion.
 * @return A handle to the output signal for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareScalar() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_signal_handle) tpt_fusion_declareOutputAsDouble(tpt_fusion_node_handle handle, 
                                                          const char *name);

/**
 * Declares a scalar parameter with the given name on the fusion, in order to 
 * be able to access it during the fusion session.
 *
 * @param handle
 *          The handle of the calling node.
 * @param name 
 *          The name of the parameter.
 * @return A handle to the parameter for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareScalar() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareScalarParamAsDouble(tpt_fusion_node_handle handle, 
                                                              const char *name);

/**
 * Declares an array parameter with the given name on the fusion.
 *
 * @param handle
 *          The handle of the calling node.
 * @param name 
 *          The name of the parameter.
 * @param size
 *          The number of elements to store within the array.
 * @return A handle to the parameter for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareArray() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareArrayParamAsDouble(tpt_fusion_node_handle handle, 
                                                             const char *name, unsigned int size);

/**
 * Declares a node's table variable on the FUSION and returns a handle that allows 
 * to read and write that variable during a FUSION session. 
 *
 * In order to define a table's keys (axis), an array variable must be provided. 
 * If this array of keys is anonymous (has no name), a new array variable must 
 * be declared using the naming scheme "<tablename>:KEYS" (e.g. "MYTABLE:KEYS"). 
 * This ensures, that all nodes use the same anonymous keys for the same table.
 *
 * In case, a variable with the given name has already been declared, it will be 
 * returned, if its type and role match the declaration, otherwise, an error will 
 * be set. 
 *
 * @param handle
 *          The handle of the calling node.
 * @param name 
 *          The name of the table to declare.
 * @param keys
 *          The keys of the table to declare.
 * @param role
 *          The role of the table to declare. 
 *
 * @return A handle to the variable for accessing it during the FUSION session.
 *
 * @deprecated use tpt_fusion_declareCurve() or tpt_fusion_declareMap() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_variable_handle) tpt_fusion_declareTable(tpt_fusion_node_handle handle,
                                                   const char *name,
                                                   tpt_fusion_variable_handle keys,
                                                   tpt_fusion_variable_role role);

/**
 * Declares a table parameter with the given name on the fusion. Both, values and x-axis must be 
 * arrays. The size of the value array must match the size of the x-axis array at the moment of
 * declaration!
 *
 * @param handle
 *          The handle of the calling node.
 * @param name
 *          The name of the table parameter to declare on the fusion.
 * @param values
 *          The handle of the values array variable to be used in the table. The values array
 *          will be copied into the newly created table. Afterwards there is no relation
 *          between the values variable and the table.
 * @param xaxis
 *          The handle to an array parameter, forming the parameter's x-axis.
 * @return A handle to the parameter for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareTable() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareTableParamAsDouble(tpt_fusion_node_handle handle, 
                                                             const char *name, 
                                                             tpt_fusion_param_handle values,
                                                             tpt_fusion_param_handle xaxis);

/**
 * Declares a curve parameter with the given name on the fusion. Both, values and x-axis must be 
 * arrays. The size of the value array must match the size of the x-axis array at the moment of
 * declaration!
 *
 * @param handle
 *          The handle of the calling node.
 * @param name
 *          The name of the parameter to declare on the fusion.
 * @param values
 *          The handle of the values array variable to be used in the table. The values array
 *          will be copied into the newly created table. Afterwards there is no relation
 *          between the values variable and the table.
 * @param xaxis
 *          The handle to an array parameter, forming the parameter's x-axis.
 * @return A handle to the parameter for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareCurve() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareCurveParamAsDouble(tpt_fusion_node_handle handle, 
                                                             const char *name, 
                                                             tpt_fusion_param_handle values,
                                                             tpt_fusion_param_handle xaxis);

/**
 * same as tpt_fusion_declareCurveParamAsDouble(). Thus see tpt_fusion_declareCurveParamAsDouble() 
 * for documentation details.
 *
 * @deprecated use tpt_fusion_declareCurve() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareTable2DParamAsDouble(tpt_fusion_node_handle handle, 
                                                               const char *name, 
                                                               tpt_fusion_param_handle values,
                                                               tpt_fusion_param_handle xaxis);


/**
 * Declares a map parameter with the given name on the fusion. Values, x-axis and y-axis must be 
 * arrays. The length(values) must be equal to length(xaxis)*length(yaxis) at the moment of
 * declaration!
 *
 * @param handle
 *          The handle of the calling node.
 * @param name
 *          The name of the parameter to declare on the fusion.
 * @param xaxis
 *          The handle to an array parameter, forming the parameter's x-axis.
 * @param yaxis
 *          The handle to an array parameter, forming the parameter's y-axis.
 * @param values
 *          The handle of the values array variable to be used in the table. The values array
 *          will be copied into the newly created table. Afterwards there is no relation
 *          between the values variable and the table.
 * @return A handle to the parameter for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareMap() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareMapParamAsDouble(tpt_fusion_node_handle handle, 
                                                           const char *name, 
                                                           tpt_fusion_param_handle values,
                                                           tpt_fusion_param_handle xaxis,
                                                           tpt_fusion_param_handle yaxis);

/**
 * same as tpt_fusion_declareMapParamAsDouble(). Thus see tpt_fusion_declareMapParamAsDouble()
 * for documentation details.
 *
 * @deprecated use tpt_fusion_declareMap() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareTable3DParamAsDouble(tpt_fusion_node_handle handle, 
                                                               const char *name, 
                                                               tpt_fusion_param_handle values,
                                                               tpt_fusion_param_handle xaxis,
                                                               tpt_fusion_param_handle yaxis);

/**
 * Creates a parameter on the fusion session. A handle is returned, which allows 
 * to access the parameter axes and values during a fusion session.
 * 
 * - Creates a scalar if valuesSize is 1 and xAxisSize is 0 and yAxisSize is 0.
 * - Creates an array if valuesSize is greater than 1 and xAxisSize is 0 and yAxisSize is 0.
 * - Creates a curve if xAxisSize is greater than 0, but yAxisSize is 0.
 * - Creates a map if xAxisSize is greater than 0 and yAxisSize is greater than 0.
 *
 * @param handle
 *          The handle of the calling node.
 * @param name 
 *          The name of the parameter.
 * @param valuesSize
 *          The number of values stored with the parameter (must be greater than 0).
 * @param xAxisSize
 *          The number of values that the parameter's x-axis defines.
 * @param yAxisSize
 *          The number of values that the parameter's y-axis defines.
 *
 * @return A handle to the parameter for accessing it during the fusion session.
 *
 * @deprecated use tpt_fusion_declareScalar(), tpt_fusion_declareArray(), 
 *               tpt_fusion_declareCurve(), or tpt_fusion_declareMap() instead
 */
TPT_DECLARE_FUSION(tpt_fusion_param_handle) tpt_fusion_declareParamAsDouble(tpt_fusion_node_handle handle, 
                                                        const char *name, 
                                                        unsigned int valuesSize,
                                                        unsigned int xAxisSize,
                                                        unsigned int yAxisSize);

/**
 * Returns the current value of a scalar variable.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The handle of the variable to be read.
 * @return The current value of the variable during the fusion session.
 *
 * @deprecated use tpt_fusion_readValue() instead
 */
TPT_DECLARE_FUSION(const double) tpt_fusion_readInputAsDouble(tpt_fusion_node_handle handle, 
                                          tpt_fusion_signal_handle vhandle);

/**
 * Returns the current value of a scalar variable.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The handle of the variable to be written.
 * @param value
 *          The value to write
 *
 * @deprecated use tpt_fusion_writeValue() instead
 */
TPT_DECLARE_FUSION(void) tpt_fusion_writeOutputAsDouble(tpt_fusion_node_handle handle, 
                                    tpt_fusion_signal_handle vhandle,
                                    const double value);

/**
 * Returns the given signal's name.
 *
 * @deprecated use tpt_fusion_getVariableName() instead
 */
TPT_DECLARE_FUSION(const char *) tpt_fusion_getSignalName(tpt_fusion_node_handle handle, 
                                     tpt_fusion_signal_handle vhandle);


/**
 * Returns the given parameter's name.
 *
 * @deprecated use tpt_fusion_getVariableName() instead
 */
TPT_DECLARE_FUSION(const char *) tpt_fusion_getParamName(tpt_fusion_node_handle handle, 
                                    tpt_fusion_param_handle vhandle);

/**
 * Returns a value not equal to zero, iff the given variable is a 
 * table, curve or map
 *
 * @deprecated use tpt_fusion_getVariableType() instead
 */
TPT_DECLARE_FUSION(unsigned int) tpt_fusion_isComplexParam(tpt_fusion_node_handle handle, 
                                       tpt_fusion_param_handle vhandle);

/**
 * Returns the values defined by the given parameter's x-axis. The result is read-only.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @return the values of the given parameter's x-axis.
 *
 * @deprecated use tpt_fusion_getVariableXAxis() and tpt_fusion_readValues() instead
 */
TPT_DECLARE_FUSION(const double *) tpt_fusion_getParamXAxisValuesAsDoubles(tpt_fusion_node_handle handle, 
                                                      tpt_fusion_param_handle vhandle);

/**
 * Sets the values defined by the given parameter's x-axis. 
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @param axisValues
 *          The values to set for the x-axis. The number of values is taken from the xaxis, so
 *          ensure that the xaxis has the correct length before calling this function
 *
 * @deprecated use tpt_fusion_getVariableXAxis() and tpt_fusion_writeValues() instead
 */
TPT_DECLARE_FUSION(void) tpt_fusion_setParamXAxisValuesAsDoubles(tpt_fusion_node_handle handle, 
                                             tpt_fusion_param_handle vhandle,
                                             const double *axisValues);

/**
 * Returns the number of values the x-axis of the given parameter defines.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @return the number of values the x-axis of the given parameter defines.
 *
 * @deprecated use tpt_fusion_getVariableXAxis() and tpt_fusion_getVariableLength() instead
 */
TPT_DECLARE_FUSION(unsigned int) tpt_fusion_getParamXAxisSize(tpt_fusion_node_handle handle, 
                                 tpt_fusion_param_handle vhandle);

/**
 * Returns the values defined by the given parameter's y-axis. The result is read-only.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @return the values of the given parameter's y-axis.
 *
 * @deprecated use tpt_fusion_getVariableYAxis() and tpt_fusion_readValues() instead
 */
TPT_DECLARE_FUSION(const double *) tpt_fusion_getParamYAxisValuesAsDoubles(tpt_fusion_node_handle handle, 
                                                      tpt_fusion_param_handle vhandle);

/**
 * Sets the values defined by the given parameter's y-axis. 
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @param axisValues
 *          The values to set for the y-axis. The number of values is taken from the yaxis, so
 *          ensure that the yaxis has the correct length before calling this function
 *
 * @deprecated use tpt_fusion_getVariableYAxis() and tpt_fusion_writeValues() instead
 */
TPT_DECLARE_FUSION(void) tpt_fusion_setParamYAxisValuesAsDoubles(tpt_fusion_node_handle handle, 
                                             tpt_fusion_param_handle vhandle,
                                             const double *axisValues);

/**
 * Returns the number of values the y-axis of the given parameter defines.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @return the number of values the y-axis of the given parameter defines.
 *
 * @deprecated use tpt_fusion_getVariableYAxis() and tpt_fusion_getVariableLength() instead
 */
TPT_DECLARE_FUSION(unsigned int) tpt_fusion_getParamYAxisSize(tpt_fusion_node_handle handle, 
                                 tpt_fusion_param_handle vhandle);

/** 
 * Returns the values of a given parameter. The result is read-only. Map values 
 * will be stored "x-axis first", i.e. the index is (x_i + n * y_i) where n is 
 * the x-axis size.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @return the values, the given parameter stores.
 *
 * @deprecated use tpt_fusion_readValues() instead
 */
TPT_DECLARE_FUSION(const double *) tpt_fusion_getParamValuesAsDoubles(tpt_fusion_node_handle handle, 
                                                 tpt_fusion_param_handle vhandle);

/**
 * Sets the values defined by the given parameter's values. 
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @param values
 *          The values to set for the parameter's values. The number of values is 
 *          taken from the current value, so ensure that the variable has the 
 *          correct length before calling this function
 *
 * @deprecated use tpt_fusion_writeValues() instead
 */
TPT_DECLARE_FUSION(void) tpt_fusion_setParamValuesAsDoubles(tpt_fusion_node_handle handle, 
                                        tpt_fusion_param_handle vhandle,
                                        const double *values);

/**
 * Returns the number of values defined by the given parameter. 
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The parameter's handle.
 * @return the amount of values the given parameter stores.
 *
 * @deprecated use tpt_fusion_getVariableLength() instead
 */
TPT_DECLARE_FUSION(unsigned int) tpt_fusion_getParamValuesSize(tpt_fusion_node_handle handle, 
                                           tpt_fusion_param_handle vhandle);


/**
 * Returns a read-only pointer to the current value of the given variable. If the 
 * given variable is a map, curve or table, a pointer to its values is returned.
 *
 * In order to read an axis of a map, curve or table, one may retrieve them with
 * a call to 'tpt_fusion_getVariableXAxis()' or 'tpt_fusion_getVariableYAxis()'.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The handle of the variable to read. 
 *
 * @return A read-only pointer to the current value of the given variable. 
 *
 * @deprecated use tpt_fusion_readValues() instead
 */
TPT_DECLARE_FUSION(const double *) tpt_fusion_read(tpt_fusion_node_handle handle, 
                              tpt_fusion_variable_handle vhandle);

/**
 * Writes a new value to the given variable on the fusion. If the given variable 
 * is a map, curve or table, this function will always write the values.
 *
 * The number of values copied from the given value pointer depends on the given
 * variable's length.
 * 
 * In order to write an axis of a map, curve or table, one may retrieve them with
 * a call to 'tpt_fusion_getVariableXAxis()' or 'tpt_fusion_getVariableYAxis()'.
 *
 * @param handle
 *          The handle of the calling node.
 * @param vhandle
 *          The handle of the variable to write. 
 * @param value
 *          A pointer from which the variable's new value will be copied from. 
 *
 * @deprecated use tpt_fusion_writeValues() instead
 */
TPT_DECLARE_FUSION(void) tpt_fusion_write(tpt_fusion_node_handle handle, 
                      tpt_fusion_variable_handle vhandle, 
                      const double *value);

#ifdef __cplusplus
}
#endif

#endif /* __TPT_FUSION_OBSOLETE_H_ */
