/*********************************************************************
 *              Copyright (c) 2021 PikeTec GmbH
 *               CONFIDENTIAL AND PROPRIETARY.
 *                   ALL RIGHTS RESERVED
 *              - PROPRIETARY OF PikeTec GmbH -
 **********************************************************************/
/* clang-format off */

/**
 * declares all tracking operations for code coverage and C-code instrumentation
 */

#ifndef __TPT_INSTRUMENTATION_H_
#define __TPT_INSTRUMENTATION_H_

#include "TPT_Types.h"

/* version macros, used by instrumented test frame and passed to TPT_TRACK_init_v2 */
#define TPT_TRACK_VERSION_TPT18 1 

#ifdef _MSC_VER 
/* ignore all warnings regarding unused functions for visual studio compilers */
#pragma warning(push)
#pragma warning( disable : 4505 )
#endif

/* TPT_Types.h does not contain a type for uint64 */
typedef unsigned long long tpt_instr_uint64;

#ifdef __cplusplus
extern "C"
{
#endif

/* 
 * init track data storage
*/
TPT_DECLARE(void) TPT_TRACK_init_v2(tpt_uint32 decisionsCount, tpt_bool fullInstr, tpt_uint32 version);

/*
 * store the tracked coverage data to file path set via the environment variable TPT_TRACK_FILE
*/
TPT_DECLARE(void) TPT_TRACK_save(void);

/*
 * store the tracked coverage data to the given file path 
 */
TPT_DECLARE(void) TPT_TRACK_save_to(const char *path);

/* 
 * cleanup track data storage (to be ready for a new run)
*/
TPT_DECLARE(void) TPT_TRACK_cleanup(void);

/* 
 * temporary store values, needed for test data generation with TASMO
*/
TPT_DECLARE(tpt_uint8) TPT_TRACK_push_uint8(tpt_uint8 value);

TPT_DECLARE(tpt_int8) TPT_TRACK_push_int8(tpt_int8 value);

TPT_DECLARE(tpt_uint16) TPT_TRACK_push_uint16(tpt_uint16 value);

TPT_DECLARE(tpt_int16) TPT_TRACK_push_int16(tpt_int16 value);

TPT_DECLARE(tpt_uint32) TPT_TRACK_push_uint32(tpt_uint32 value);

TPT_DECLARE(tpt_int32) TPT_TRACK_push_int32(tpt_int32 value);

TPT_DECLARE(tpt_int64) TPT_TRACK_push_int64(tpt_int64 value);

TPT_DECLARE(tpt_instr_uint64) TPT_TRACK_push_uint64(tpt_instr_uint64 value);

TPT_DECLARE(tpt_float) TPT_TRACK_push_float(tpt_float value);

TPT_DECLARE(tpt_double) TPT_TRACK_push_double(tpt_double value);

TPT_DECLARE(tpt_int32) TPT_TRACK_lt(tpt_int32 conditionRes, tpt_uint32 conditionId);
  
TPT_DECLARE(tpt_int32) TPT_TRACK_le(tpt_int32 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_int32) TPT_TRACK_gt(tpt_int32 conditionRes, tpt_uint32 conditionId);
  
TPT_DECLARE(tpt_int32) TPT_TRACK_ge(tpt_int32 conditionRes, tpt_uint32 conditionId);
    
TPT_DECLARE(tpt_int32) TPT_TRACK_eq(tpt_int32 conditionRes, tpt_uint32 conditionId);  
  
TPT_DECLARE(tpt_int32) TPT_TRACK_neq(tpt_int32 conditionRes, tpt_uint32 conditionId);
  
TPT_DECLARE(tpt_int8) TPT_TRACK_condition_int8(tpt_int8 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_int16) TPT_TRACK_condition_int16(tpt_int16 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_int32) TPT_TRACK_condition_int32(tpt_int32 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_int64) TPT_TRACK_condition_int64(tpt_int64 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_uint8) TPT_TRACK_condition_uint8(tpt_uint8 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_uint16) TPT_TRACK_condition_uint16(tpt_uint16 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_uint32) TPT_TRACK_condition_uint32(tpt_uint32 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_instr_uint64) TPT_TRACK_condition_uint64(tpt_instr_uint64 conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_float) TPT_TRACK_condition_float(tpt_float conditionRes, tpt_uint32 conditionId);

TPT_DECLARE(tpt_double) TPT_TRACK_condition_double(tpt_double conditionRes, tpt_uint32 conditionId);
  
TPT_DECLARE(void) TPT_TRACK_push_and(void);

TPT_DECLARE(tpt_int32) TPT_TRACK_push_and_T(void);

TPT_DECLARE(void) TPT_TRACK_push_or(void);

TPT_DECLARE(tpt_int32) TPT_TRACK_push_or_T(void);

TPT_DECLARE(void) TPT_TRACK_push_xor(void);

TPT_DECLARE(tpt_int32) TPT_TRACK_push_xor_T(void);

TPT_DECLARE(tpt_int32) TPT_TRACK_logop(tpt_int32 conditionRes);
  
TPT_DECLARE(tpt_int32) TPT_TRACK_not(tpt_int32 conditionRes);

TPT_DECLARE(void) TPT_TRACK_init_decision(tpt_uint32 decisionId, tpt_uint32 numConditions);

TPT_DECLARE(tpt_int32) TPT_TRACK_init_decision_T(tpt_uint32 decisionId, tpt_uint32 numConditions);

TPT_DECLARE(tpt_int8) TPT_TRACK_decision_int8(tpt_int8 conditionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_int16) TPT_TRACK_decision_int16(tpt_int16 conditionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_int32) TPT_TRACK_decision_int32(tpt_int32 conditionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_int64) TPT_TRACK_decision_int64(tpt_int64 conditionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_instr_uint64) TPT_TRACK_decision_uint64(tpt_instr_uint64 decisionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_uint8) TPT_TRACK_decision_uint8(tpt_uint8 decisionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_uint16) TPT_TRACK_decision_uint16(tpt_uint16 decisionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_uint32) TPT_TRACK_decision_uint32(tpt_uint32 decisionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_instr_uint64) TPT_TRACK_decision_uint64(tpt_instr_uint64 decisionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_float) TPT_TRACK_decision_float(tpt_float decisionRes, tpt_uint32 decisionId);

TPT_DECLARE(tpt_double) TPT_TRACK_decision_double(tpt_double decisionRes, tpt_uint32 decisionId);

/* 
 * switch case statements
*/
TPT_DECLARE(tpt_int8) TPT_TRACK_switch_int8(tpt_int8 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_int16) TPT_TRACK_switch_int16(tpt_int16 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_int32) TPT_TRACK_switch_int32(tpt_int32 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_int64) TPT_TRACK_switch_int64(tpt_int64 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_uint8) TPT_TRACK_switch_uint8(tpt_uint8 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_uint16) TPT_TRACK_switch_uint16(tpt_uint16 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_uint32) TPT_TRACK_switch_uint32(tpt_uint32 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(tpt_instr_uint64) TPT_TRACK_switch_uint64(tpt_instr_uint64 switchVal, tpt_uint32 decisionId, tpt_uint32 nCases);

TPT_DECLARE(void) TPT_TRACK_case(tpt_uint32 decisionId, tpt_uint32 caseId);

/* functions for data type size_t
   since this type is compiler-dependant these functions are defined here as static functions
   This type is usually defined in stddef.h, check for common makros before defining functions
   */
#if defined(_SIZE_T_) || defined(_SIZE_T) || defined(_STDDEF_H) || defined(_STDDEF_H_) || defined(_ANSI_STDDEF_H) \
     || defined(__STDDEF_H__)
#if defined(__GNUC__) || defined(__MINGW32__) || defined(__MINGW64__) || defined(__STDC_VERSION__) /*only for mingw/msvc*/

/* suppress unused warnings for MinGW Compilers */
#if defined(__GNUC__) || defined(__MINGW32__) || defined(__MINGW64__)
static size_t TPT_TRACK_push_size_t(size_t value)__attribute__ ((unused));
static size_t TPT_TRACK_condition_size_t(size_t conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static size_t TPT_TRACK_decision_size_t(size_t decisionRes, tpt_uint32 decisionId)__attribute__ ((unused));
#endif

static size_t TPT_TRACK_push_size_t(size_t value){
  TPT_TRACK_push_uint64((tpt_instr_uint64)value);
  return value;
}
static size_t TPT_TRACK_condition_size_t(size_t conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_condition_uint64(conditionRes, conditionId);
  return conditionRes;
}

static size_t TPT_TRACK_decision_size_t(size_t decisionRes, tpt_uint32 decisionId){
  TPT_TRACK_decision_uint64(decisionRes, decisionId);
  return decisionRes;
}

#endif
#endif

#if ((defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L) || defined(__GNUC__) || (defined(_MSC_VER) && _MSC_VER >= 1800)) && !defined(__cplusplus)
/* 
 * Since C99 a native boolean type "_Bool" is available for C.
 * MinGW compilers support this type if std is set to C99 or newer or gnu extensions are available.
 * MSVC supports this type since Visual Studio 13 (_MSC_VER = 1800).
 * For Instrumentation we define the proper functions here (and map them to the corresponding functions for uint8).
*/  

/* suppress unused warnings for MinGW Compilers */
#if defined(__GNUC__) || defined(__MINGW32__) || defined(__MINGW64__)
static _Bool TPT_TRACK_push_bool(_Bool value)__attribute__ ((unused));
static _Bool TPT_TRACK_condition_bool(_Bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static _Bool TPT_TRACK_decision_bool(_Bool decisionRes, tpt_uint32 decisionId)__attribute__ ((unused));
#endif

static _Bool TPT_TRACK_push_bool(_Bool value){
  TPT_TRACK_push_uint8(value?1:0);
  return value;
}

static _Bool TPT_TRACK_condition_bool(_Bool conditionRes, tpt_uint32 conditionId) {
  TPT_TRACK_condition_uint8(conditionRes?1:0, conditionId);
  return conditionRes;
}

static _Bool TPT_TRACK_decision_bool(_Bool decisionRes, tpt_uint32 decisionId) {
  TPT_TRACK_decision_uint8(decisionRes?1:0, decisionId);
  return decisionRes;
}
#endif

#ifdef __cplusplus
}
#endif

/* wrapper functions for instrumentation in *.cpp files */
#ifdef __cplusplus
/* surpress unused warnings for MinGW Compilers */
#if defined(__GNUC__) || defined(__MINGW32__) || defined(__MINGW64__)
static bool TPT_TRACK_lt_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static bool TPT_TRACK_le_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static bool TPT_TRACK_gt_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static bool TPT_TRACK_ge_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static bool TPT_TRACK_eq_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static bool TPT_TRACK_neq_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));

static bool TPT_TRACK_logop_CPP(bool conditionRes)__attribute__ ((unused));
static bool TPT_TRACK_not_CPP(bool conditionRes)__attribute__ ((unused));

static bool TPT_TRACK_push_bool_CPP(bool value)__attribute__ ((unused));
static bool TPT_TRACK_condition_bool_CPP(bool conditionRes, tpt_uint32 conditionId)__attribute__ ((unused));
static bool TPT_TRACK_decision_bool_CPP(bool decisionRes, tpt_uint32 decisionId)__attribute__ ((unused));

template <typename T> T TPT_TRACK_push_template_CPP(T value)__attribute__ ((unused));
#endif

/* relational op functions for c++ */
static bool TPT_TRACK_lt_CPP(bool conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_lt(conditionRes?1:0, conditionId);
  return conditionRes;
}

static bool TPT_TRACK_le_CPP(bool conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_le(conditionRes?1:0, conditionId);
  return conditionRes;
}

static bool TPT_TRACK_gt_CPP(bool conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_gt(conditionRes?1:0, conditionId);
  return conditionRes;
}

static bool TPT_TRACK_ge_CPP(bool conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_ge(conditionRes?1:0, conditionId);
  return conditionRes;
}

static bool TPT_TRACK_eq_CPP(bool conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_eq(conditionRes?1:0, conditionId);
  return conditionRes;
}

static bool TPT_TRACK_neq_CPP(bool conditionRes, tpt_uint32 conditionId){
  TPT_TRACK_neq(conditionRes?1:0, conditionId);
  return conditionRes;
}

/* logical operator handling for c++ */
static bool TPT_TRACK_logop_CPP(bool conditionRes){
  TPT_TRACK_logop(conditionRes?1:0);
  return conditionRes;
}

static bool TPT_TRACK_not_CPP(bool conditionRes){
  TPT_TRACK_not(conditionRes?1:0);
  return conditionRes;
}

static bool TPT_TRACK_push_bool_CPP(bool value){
  TPT_TRACK_push_uint8(value?1:0);
  return value;
}

static bool TPT_TRACK_condition_bool_CPP(bool conditionRes, tpt_uint32 conditionId) {
  TPT_TRACK_condition_uint8(conditionRes?1:0, conditionId);
  return conditionRes;
}

static bool TPT_TRACK_decision_bool_CPP(bool decisionRes, tpt_uint32 decisionId) {
  TPT_TRACK_decision_uint8(decisionRes?1:0, decisionId);
  return decisionRes;
}

template <typename T> 
static T TPT_TRACK_push_template_CPP(T value){
  TPT_TRACK_push_double((double)value); /* push as double */
  return value;
}

#endif

/* DEPRECATED, do not use, use TPT_TRACK_init_v2 instead */
TPT_DECLARE(void) TPT_TRACK_init(tpt_uint32 decisionsCount, tpt_bool fullInstr);

#ifdef _MSC_VER 
#pragma warning(pop)
#endif

#endif /* __TPT_INSTRUMENTATION_H_ */
