/*********************************************************************
 *              Copyright (c) 2021 PikeTec GmbH
 *               CONFIDENTIAL AND PROPRIETARY.
 *                   ALL RIGHTS RESERVED
 *              - PROPRIETARY OF PikeTec GmbH -
 **********************************************************************/
/* clang-format off */

#ifndef __TPT_FUSION_NODE_H_
#define __TPT_FUSION_NODE_H_

#include "tpt_fusion_base.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * These general node return codes are
 * used in order to indicate the result
 * of a call to a node's function.
 */ 
typedef enum _tpt_fusion_node_result_enum {
  TPT_FUSION_NODE_RESULT_OK,
  TPT_FUSION_NODE_RESULT_ERROR,
  TPT_FUSION_NODE_RESULT_FINISHED
} tpt_fusion_node_result;


/**
 * Implementations of this function must publish the node's functional 
 * interface to the fusion using the following fusion functions:
 *
 *    - tpt_fusion_declareScalar(..)
 *    - tpt_fusion_declareArray(..)
 *    - tpt_fusion_declareTable(..)
 *    - tpt_fusion_declareCurve(..)
 *    - tpt_fusion_declareMap(..)
 *
 * @param handle
 *          The node's associated handle.
 * @param argc
 *          The node's argument count.
 * @param argv
 *          The node's argument values.
 */
TPT_DECLARE_FUSION_NODE(tpt_fusion_node_result) tpt_fusion_node_startup(tpt_fusion_node_handle handle, int argc, char **argv);

/**
 * This function is called by the fusion in order to give the node a chance 
 * to initialize the logical function it hosts (e.g. set default values...).
 *
 * @param handle
 *          The node's associated handle.
 */
TPT_DECLARE_FUSION_NODE(tpt_fusion_node_result) tpt_fusion_node_initfcn(tpt_fusion_node_handle handle);

/**
 * This function is called with every time step in order to run the node's
 * logical function it hosts.
 *
 * @param handle
 *          The node's associated handle.
 */
TPT_DECLARE_FUSION_NODE(tpt_fusion_node_result) tpt_fusion_node_callfcn(tpt_fusion_node_handle handle);

/**
 * This function is called when the node is disconnected from the fusion.
 * It gives the node a chance to shutdown in a clean way.
 *
 * @param handle
 *          The node's associated handle.
 */
TPT_DECLARE_FUSION_NODE(tpt_fusion_node_result) tpt_fusion_node_shutdown(tpt_fusion_node_handle handle);

#ifdef __cplusplus
}
#endif

#endif /* __TPT_FUSION_NODE_H_ */
