/*********************************************************************
|  ----------------------------------------------------------------
|              Copyright (c) 2021 PikeTec GmbH
|  CONFIDENTIAL AND PROPRIETARY.ALL RIGHTS RESERVED -PROPRIETARY OF
|                       PikeTec GmbH
|********************************************************************/
#ifndef TPT_SFUN_INLINED_H
#define TPT_SFUN_INLINED_H

#include "TPT_Types.h"

/* Marker for struct element in dims hirachy */
#define DIMS_STRUCT_ELEM 0xFFFFFFFF

typedef enum {
  tpt_not_initialized = 0, /* Status: TPT noch nicht initiliasiert */
  tpt_initialized     = 1, /* Status: Die TPT SFunction ist voll ausspezifiziert und ihr Interface steht fest in tpt_io */
  tpt_running         = 2, /* Status: Die Simulation laeuft */
  tpt_end             = 3, /* Status: Die Simulation ist fertig abgearbeitet */
  tpt_error           = 4  /* Status: Es ist ein Fehler aufgetreten */
} tpt_sfunction_status;

typedef enum {
  tpt_stop            = 0, /* Kommando: Auf den Ausgangssignalen werden die Standardwerte der SFunction geschrieben */
  tpt_initialize      = 1, /* Kommando: Byte-Daten liegen vor und koennen eingelesen werden. Solange die Daten noch */
                           /* nicht eingelesen sind werden die Standardwerte der SFunction auf die Ausgangskanaele geschrieben */
                           /* Wenn die Daten eingelesen sind werden die dort spezifizierten Standardwerte benutzt. */
  tpt_run             = 2, /* Kommando: Simulation laeuft */
  tpt_direct_start    = 3  /* Kommando: Direkter Start der Testausführung, die Umgebung ist vollstündig initialisiert */
} tpt_sfunction_command;


typedef struct {
  char*           name;                 /* Vollständiger Name des signals, Bus-Ebenen sind dabei mit "." voneinander getrennt */
  tpt_int32       mlBusElementOffset;   /* Offset für die addresse des ersten array-elements dieses signals in der MATLAB Schnittstelle */
  tpt_uint16      nSlTypes;             /* Anzahl der daten-typen in der typ-hierarchie */
  tpt_uint32*     slTypeIds;            /* SL type Ids in der typ-hierarchie, wird nur im SIL verwendet */
  tpt_uint32*     slTypeSizes;          /* Daten-typ-Größen in bytes in der typ-hierarchie */
  tpt_uint8       nDims;                /* Anzahl der dimensionen und struct-ebenen */
  tpt_tsize*      dims;                 /* Dimensionen und struct-ebenene in der signal-hierachie (für struct-ebenenen liegt hier der value UINT32_MAX_VALUE an */
  tpt_basetype    dataType;             /* Vom primitiven MATLAB Datentyp abgeleiteter (erwarteter) TPT-Datentyp */
  tpt_bool        isScaledFixedPointType; /* Ist der simulink datentyp ein skalierter fixed point type? */
  tpt_uint16      tpt_vm_adr;           /* Adresse(Index) des Kanals in der TPT-VM */
  tpt_tsize       nMlElementOffsets;    /* Anzahl der einzel-Addressen der array-elemente am port, dies ist 0, wenn wenn diese einzel-addressen nicht benötigt werden */
  tpt_uint32*     mlElementOffsets;     /* Array mit einzel-Addressen der array-elemente am port, dies ist NULL, wenn wenn diese einzel-addressen nicht benötigt werden */
} tpt_port_signal;

typedef struct {
  void*           portBaseAddress;      /* Basis-Addresse für diesen Port im MATLAB Interface */
  tpt_uint16      nPortSignals;         /* Anzahl der Einzel-Signale für diesen Port */
  tpt_port_signal *signals;             /* Einzel-Signale für diesen Port */
  tpt_bool        isNonVirtualBus;      /* Ist das signal dieses ports ein non-virtual-bus? */
  tpt_bool        writeInitialValue;    /* Soll direkt bei der initialisierung der initiale value rausgeschrieben werden? */
  tpt_bool        dataInitialized;      /* Sind die daten in dieser struktur bereits initialisiert? */
} tpt_port_data;

typedef struct {
  char*           name;                 /* Name der Server Function */
  tpt_uint16      tpt_vm_adr;           /* Adresse(Index) der Function in der TPT-VM */
  tpt_uint16      nFcnInArgs;           /* Anzahl Input Argumente */
  tpt_port_data*  fcnInArgs;            /* Daten für die Input Argumente */
  tpt_uint16      nFcnOutArgs;          /* Anzahl Output Argumente */
  tpt_port_data*  fcnOutArgs;           /* Daten für die Output Argumente */
} tpt_fcn_data;
  
typedef struct {
  tpt_uint16      tpt_ins_size;         /* Anzahl der Eingänge */
  tpt_port_data*  tpt_ins_data;         /* Port-daten für die Eingänge */
  tpt_uint16      tpt_outs_size;        /* Anzahl der Ausgänge */
  tpt_port_data*  tpt_outs_data;        /* Port-daten für die Ausgänge */
} tpt_in_out;

typedef struct {
  tpt_uint16      tpt_server_fcns_size; /* Anzahl der Server Functions */
  tpt_fcn_data*   tpt_server_fcns_data; /* Function-daten der Server Functions */
} tpt_server_functions;

#if (!defined(INT64_T))
  typedef tpt_int64 int64_T;
#endif

/* function handle definition to copy fixed point port data, used to register function handle from S-function code */
typedef void (*tptCopyFixdtPortDataFunction)(void *simStructPtr, tpt_uint32 portIdx, tpt_uint32 elemIdx, 
  void *tptVMAddress, tpt_basetype vmType, tpt_uint8 vmByteSize, void *mlAddress, tpt_uint32 mlTypeSize);
  
  /* function handle definition to copy fixed point function arg data, used to register function handle from S-function code */
typedef void (*tptCopyFixdtFcnArgDataFunction)(void *simStructPtr, tpt_int32 serverFcnIndex, tpt_uint32 portIdx, tpt_uint32 elemIdx, 
  void *tptVMAddress, tpt_basetype vmType, tpt_uint8 vmByteSize, void *mlAddress, tpt_uint32 mlTypeSize);
	
typedef void (*tptErrorHandler)(void *simStructPtr, char* msg);

#if defined(__cplusplus)
extern "C"
{
#endif

TPT_DECLARE(tpt_uint32) tpt_sfun_getCycleCount();

/* get the tpt_base type for an enum type defined in MATLAB via the name */
TPT_DECLARE(tpt_basetype) getEnumBaseTypeByName(const char* name);

/* Benachrichtigt Matlab mit einer Fehlermeldung. Bricht einen laufenden Testlauf in Matlab ab. */
TPT_DECLARE(void) tpt_reportError(void *simStructPtr, const char *e);       /* Fehlermeldung */

/* Test setup mit interface-daten und pfaden zu tvm und tptbin files (wird vor dem eigerntlichen start gesetzt) */
TPT_DECLARE(void) tpt_sfun_setup(const tpt_double stepsize, tptErrorHandler errorHandler, 
        const char *prog_file, const char *bin_file,
        const tpt_in_out *ioDescription,
        tptCopyFixdtPortDataFunction copyPortDataTpt2SlFcn, 
        tptCopyFixdtPortDataFunction copyPortDataSl2TptFcn);

/* Setup für function-interface data, wird vom tpt-function-handler block aufgerufen (der die definitionen für server-functions bereitstellt) */
TPT_DECLARE(void) tpt_sfun_setup_fcn_io(const tpt_server_functions *fcn_data,
        tptCopyFixdtFcnArgDataFunction copyFcnArgDataTpt2SlFcn, 
        tptCopyFixdtFcnArgDataFunction copyFcnArgDataSl2TptFcn);

/* Initialisierung des tests, macht nichts, wenn bereits initialisiert */
TPT_DECLARE(int) tpt_sfun_init(void *simStructPtr, const tpt_bool initOnlyMode);

/*
 * Trigger das schreiben der outputs aus der VM nach MATLAB
 * 
 * Falls der Test beendet ist, kommt hier 0 zurück, ansonsten 1.
 */
TPT_DECLARE(int) tpt_sfun_write(void *simStructPtr);
          
/*
 * Trigger das rechnen der VM.
 * 
 * Falls der Test beendet ist, kommt hier 0 zurück, ansonsten 1.
 */
TPT_DECLARE(int) tpt_sfun_cycle(void *simStructPtr);

/*
 * Trigger das lesen der der inputs aus MATLAB in die VM.
 * 
 * Falls der Test beendet ist, kommt hier 0 zurück, ansonsten 1.
 */
TPT_DECLARE(int) tpt_sfun_update(void *simStructPtr);

TPT_DECLARE(void) tpt_sfun_terminate();

TPT_DECLARE(int) tptCallServerFunction(void *simStructPtr, tpt_int32 serverFcnIndex);

TPT_DECLARE(void*) tpt_sfun_malloc(tpt_uint32 size);

TPT_DECLARE(void) tpt_sfun_free(const void* ptr);

/*
 * Generate C-code to statically recreate the given port data array
 */
TPT_DECLARE(void) tpt_sfun_gen_rtw_portdata(char** bufOut, const char* varName, tpt_uint32 nPortData, tpt_port_data *portData);

/*
 * Collect suffix-strings to access port-element based on port bus-types as (in array-format, in case of non-bus type one element as empty-string)
 */
TPT_DECLARE(void) tpt_sfun_gen_rtw_port_signals_suffixes(char** bufOut, tpt_port_data *portData);

/*
 * collect type names for vm types of signals, enum names of "tpt_basetype"
 */
TPT_DECLARE(void) tpt_sfun_gen_rtw_port_signals_vmtype_names(char** bufOut, tpt_port_data *portData);

/*
 * Collect all type-ids in hirachy for all sub-signals of given port, the returned pointer should be cleaned up afterwards
 */
TPT_DECLARE(void) tpt_sfun_gen_rtw_port_signals_typeids(tpt_uint32 **ids,tpt_uint32* nIds, tpt_port_data *portData);

/*
 * Cleanup internal buffer created during other tpt_sfun_rtw_<...> methods
*/
TPT_DECLARE(void) tpt_sfun_rtw_cleanup(void);

#if defined(__cplusplus)
}
#endif

#endif /* TPT_SFUN_INLINED_H */
