% initialize the 'matlab_tpt_env'
%***********************************************************************
%  Copyright (c) PikeTec GmbH
%***********************************************************************

% remember that we already loaded the environment...
tptenv.path = fileparts(which(mfilename));

disp('init_tpt_env> start ...')
disp(['init_tpt_env> path:       ' tptenv.path]);

% cleanup MATLAB path
% detect paths to any TPT versions by regexp ([^;]*\\tptplugins\\[^;]*)
tpt_tmp_paths = regexp(path(), ... 
        ['([^', regexptranslate('escape', pathsep), ']*', ...
        regexptranslate('escape', [filesep, 'tptplugins', filesep]), ...
        '[^', regexptranslate('escape', pathsep), ']*)'], ...
        'match');
% remove paths to current TPT from list
tpt_tmp_paths(cellfun(@(p)strncmp(tptenv.path,p, ...
    length(tptenv.path)-length(fullfile('matlab','matlab_tpt_env'))), tpt_tmp_paths)) = [];
if ~isempty(tpt_tmp_paths) 
    % detected paths to other TPT versions, warn and remove
    disp(['init_tpt_env> Detected ',mat2str(length(tpt_tmp_paths)), ' entries to other TPT versions in MATLAB path. They will be removed.']);
    rmpath(tpt_tmp_paths{:})
end
clear tpt_tmp_paths;

% extend the Matlab path
addpath(tptenv.path);
if (exist(fullfile(tptenv.path, 'extern', 'm'),'file') == 7)
    addpath(fullfile(tptenv.path, 'extern', 'm'));
    addpath(fullfile(tptenv.path, 'extern', 'lib'));
    addpath(fullfile(tptenv.path, 'extern', 'm', 'util'));
else
    addpath(fullfile(tptenv.path, 'extern', 'm_75'));
    addpath(fullfile(tptenv.path, 'extern', 'm_75', 'lib'));
    addpath(fullfile(tptenv.path, 'extern', 'm_75', 'util'));
end

[tpt_mlVerMaj, tpt_mlVerMin] = tptGetMatlabVersion();
if( tpt_mlVerMaj < 7 || (tpt_mlVerMaj == 7 && tpt_mlVerMin < 10))
    error('Matlab versions prior to Matlab 7.10 are no longer supported. The last TPT version that supports older versions is TPT 15');
end

% add path to mdl or slx version based on MATLAB version number
if tpt_mlVerMaj < 8
    addpath(fullfile(tptenv.path, 'extern', 'm_75', 'sllibmdl'));
else
    addpath(fullfile(tptenv.path, 'extern', 'm_75', 'sllibslx'));
end

if (exist(fullfile(tptenv.path, 'extern', 'fusion'),'file') == 7)
    addpath(fullfile(tptenv.path, 'extern', 'fusion'));
end

if (exist(fullfile(tptenv.path, 'extern', 'dashboard'),'file') == 7)
    addpath(fullfile(tptenv.path, 'extern', 'dashboard'));
end

if (exist(fullfile(tptenv.path, '..', '..', 'dspacepil', 'matlab', 'scripts'),'file') == 7)
    addpath(fullfile(tptenv.path, '..', '..', 'dspacepil', 'matlab', 'scripts'));
end

if (exist(fullfile(tptenv.path,'..', '..', 'tasmosimulink', 'io', 'matlab', 'scripts'),'file') == 7)
    addpath(fullfile(tptenv.path,'..', '..', 'tasmosimulink', 'io', 'matlab', 'scripts'));
    addpath(fullfile(tptenv.path,'..', '..', 'tasmosimulink', 'io', 'matlab', 'scripts', 'trackFcnWrapper'));
end

addpath(fullfile(tptenv.path, 'extern', 'include'));
addpath(fullfile(tptenv.path, '..', '..', '..', 'public', 'include')); % public TPT include dir


if ispc % is windows
    % For windows prepend the TPT MATLAB lib directory to system PATH
    % environment variable. With this the operating system is able to finde
    % the TPT S-Function library files.
    % Note: In case of linux the LD_LIBRARY_PATH environment variable needs
    % to be set instead. This is done by TPT when TPT starts MATLAB. If in
    % linux MATLAB is  started from outside of TPT this needs to be done on
    % system level.
    setenv('PATH', [fullfile(tptenv.path, 'extern', 'lib'), pathsep, getenv('PATH')]);
end
addpath(fullfile(tptenv.path, 'extern', 'lib'));

tpt_slashIndex = strfind(tptenv.path, filesep);
if (isempty(tpt_slashIndex))
    % Tool-Versionen feststellen
    v = ver('matlab_tpt_env');
else
    tpt_slashIndex = tpt_slashIndex(end);
    v = ver(tptenv.path(tpt_slashIndex+1:end));
end
if(length(v) > 1)
    v = v(1); % sometimes multiple answers occur in MATLAB 6.1 ...
end
clear tpt_slashIndex;

if isempty(v)
    % handle the case of an empty return value by the ver command
    v(1).Name = 'TPT Integration Toolbox';
    v.Version = '19u1';
    v.Release = '()';
    v.Date = '2023';
end

tpt_a = v.Name;
tptenv.name = tpt_a;
tpt_a = v.Version;
tptenv.version = tpt_a;
tpt_a = v.Release;
tptenv.release = tpt_a;
tpt_a = v.Date;
tptenv.date = tpt_a;
% use the following line if you want to use custom registry ports
% tptenv.ports = [ 12345, 33233];
clear v;
clear tpt_a;

% after matlab paths added but before extending java path and initializing
% compiler find script files named 'tptInitEnvBeforeStartup.m' on matlab
% path and run them
tptInitEnvBeforeStartupScripts = which('tptInitEnvBeforeStartup.m', '-all');
for tpt_tmp_i = 1:length(tptInitEnvBeforeStartupScripts)
    run(tptInitEnvBeforeStartupScripts{tpt_tmp_i});
end
clear tpt_tmp_i;
clear tptInitEnvBeforeStartupScripts;

disp(['init_tpt_env> version:    ' tptenv.version ' (' tptenv.date ')']);

switch (tpt_getHostCompiler)
    case 'LCC'
        addpath(fullfile(tptenv.path,'extern', 'lib', 'instrlcc32'));
        setenv('PATH', [fullfile(tptenv.path,'extern', 'lib', 'instrlcc32'), ';', getenv('PATH')]);
        disp('init_tpt_env> compiler:   LCC');
        tptenv.compiler = 'LCC';
    case 'MSVC'
        addpath(fullfile(tptenv.path,'extern', 'obj', 'msvc_objs'));
        addpath(fullfile(tptenv.path,'extern', 'lib', 'instr32'));
        setenv('PATH', [fullfile(tptenv.path,'extern', 'lib', 'instr32'), ';', getenv('PATH')]);
        disp('init_tpt_env> compiler:   MSVC');
        tptenv.compiler = 'MSVC';
    case 'MSVCx64'
        addpath(fullfile(tptenv.path,'\extern\obj\msvcx64_objs'));
        addpath(fullfile(tptenv.path,'\extern\lib\instr64'));
        setenv('PATH', [fullfile(tptenv.path,'extern', 'lib', 'instr64'), ';', getenv('PATH')]);
        disp('init_tpt_env> compiler:   MSVC 64bit');
        tptenv.compiler = 'MSVC';
    case 'MSVC14x64'
        disp('init_tpt_env> compiler:   MSVC14 64bit');
        addpath([tptenv.path,'\extern\lib\instr64']);
        setenv('PATH', [fullfile(tptenv.path,'extern', 'lib', 'instr64'), ';', getenv('PATH')]);
        tptenv.compiler = 'MSVC';
    case 'MINGWx64'
        disp('init_tpt_env> compiler:   MINGW 64bit');
        addpath(fullfile(tptenv.path,'extern', 'lib', 'instr64'));
        setenv('PATH', [fullfile(tptenv.path,'extern', 'lib', 'instr64'), ';', getenv('PATH')]);
        tptenv.compiler = 'MINGW';
    case 'MINGW_LINUXx64'
        disp('init_tpt_env> compiler:   MINGW Linux 64bit');
        tptenv.compiler = 'MINGW_LINUX';
    otherwise
        disp('init_tpt_env> compiler:   **** Unknown host compiler. Please use "mex -setup" in order to determine the compiler to use!')
end

% add MatlabAccessServer to java classpath is this is not yet done
tptenv.jarpath = fullfile(tptenv.path, 'matlab_tpt_env.jar');
if ~exist('com.piketec.util.matlab.MatlabAccessServer', 'class') && exist('javaaddpath','file')
    url = java.io.File(tptenv.jarpath);
    url = url.getCanonicalPath;
    url = char(url);
    javaaddpath(url);
    disp(['init_tpt_env> Added "', url, '" to java path dynamically.']);
    clear url;
end

% find script files named 'tptInitEnvStartup.m' on matlab path and run them
tptInitEnvStartupScripts = which('tptInitEnvStartup.m', '-all');
for tpt_tmp_i = 1:length(tptInitEnvStartupScripts)
    run(tptInitEnvStartupScripts{tpt_tmp_i});
end
clear tpt_tmp_i;
clear tptInitEnvStartupScripts;

if ~isfield(tptenv, 'normi') && ~isfield(tptenv, 'notcp')
    if ~exist('com.piketec.util.matlab.MatlabAccessServer', 'class')
        % class still not in path
        disp(strcat(['init_tpt_env> TCP server: **** Cannot locate "matlab_tpt_env.jar" in MATLAB''s Java path.', ...
            13, ' Do either', ...
            13, '   (1) call "javaaddpath(''', tptenv.jarpath, ''')" or ', ...
            13, '   (2) call "edit classpath.txt" and add the path "', tptenv.jarpath, '" statically to your MATLAB installation', ...
            13, ' and restart MATLAB afterwards.']));
    else
        if isfield(tptenv, 'ports')
            % custom ports
            for tpt_tmp_i = 1:length(tptenv.ports)
                com.piketec.util.matlab.MatlabAccessServer.usePort(java.lang.Integer(tptenv.ports(tpt_tmp_i)));
            end
            clear tpt_tmp_i;
        end
        % the matlab access server is alread in class path (in classpath.txt)
        tptenv.server = com.piketec.util.matlab.MatlabAccessServer.startServer(tpt_mlVerMaj, tpt_mlVerMin);
        tptenv.port = tptenv.server.getPort;
        disp(['init_tpt_env> TCP server: started (port ', num2str(tptenv.port), ').'])
    end
else
    disp('init_tpt_env> TCP server: not started (COM connection).')
end

clear tpt_mlVerMaj;
clear tpt_mlVerMin;

disp('init_tpt_env> ... end ')
% tptenv.server.allowLogging;
