/*
 * TPT Tutorial Example
 * copyright (c) PikeTec GmbH 2016
 */
 
#include "lights_control_no_scaling.h"

#include <stdio.h>

/* constants */
const static double STEPSIZE = 0.01;

double HYSTERESE_TIME_ON = 2 ;   
double HYSTERESE_TIME_OFF = 3 ;  
double MIN_LIGHT_ON = 60 ;        
double MIN_LIGHT_OFF = 70 ;    
uint8 AUTO_MODE = 2;       
uint8 ON_MODE = 1;       

/* local variables */
static uint16 on_counter = 0;
static uint16 off_counter = 0;
static uint8  light_switch_old = 0xff;

// INPUTS:
uint8  light_switch;
double light_intensity; // scaled with slope=2^-9, bias=0

// OUTPUTS:
boolean headlight;


/*
 * Returns the light value calculated by light_switch and light_intensity value.
 */
void lights_control_cycle() {	
	uint8 old = light_switch_old;
	light_switch_old = light_switch;

	/* (1) OFF */
	if(light_switch == 0) {
		headlight = 0; 
		return;
	}
	
	/* (2) ON */
	if(light_switch == ON_MODE)	{
		headlight = 1; 
		return;
	}

	/* (3) turned to AUTO now */
	if(light_switch == AUTO_MODE && old != AUTO_MODE) {
		on_counter = 0;
		off_counter = 0;
		headlight = (light_intensity <= MIN_LIGHT_OFF); /* turn on if is 70% or below */
		return;
	}

	/* (4) in AUTO (in AUTO before already) */
	if(light_switch == AUTO_MODE && old == AUTO_MODE) {
		if(headlight == 0 && light_intensity < MIN_LIGHT_ON) {			
			if(on_counter >= HYSTERESE_TIME_ON/STEPSIZE) { 
				/* turn ON */
				on_counter = 0;
				headlight = 1;
			} else {
				++ on_counter;
				off_counter = 0;
			}
		}
		else if(headlight == 1 && light_intensity > MIN_LIGHT_OFF) {			
			if(off_counter >= HYSTERESE_TIME_OFF/STEPSIZE) {
				/* turn OFF */
				off_counter = 0;
				headlight = 0;
			} else {
				++ off_counter;
				on_counter = 0;
			}
		}
		else {
			on_counter = 0;
			off_counter = 0;
		}
	}
	
}
