package com.etas.ascet.api;

import java.util.concurrent.CompletableFuture;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.lsp4j.jsonrpc.services.JsonRequest;
import org.eclipse.lsp4j.jsonrpc.services.JsonSegment;

import com.etas.ascet.api.v1.AscetApiV1;

/**
 * For first connection to ASCET-DEVELOPER.
 * Make the decision which API version to use for further communication.
 * Use this interface in pair with {@link ClientApiGeneric}.
 * <p/>
 * <ol>
 * <li>Connect with {@link AscetApiGeneric} and query the supported versions with {@link #genericGetAscetDetails()}</li>
 * <li>Based on Client possibilities decide if a connection is possible and with which version</li>
 * <li>Reconnect with the version interface, e.g. {@link AscetApiV1}</li>
 * </ol>
 */
@NonNullByDefault
@JsonSegment("com.etas.ascet.api.server-generic")
public interface AscetApiGeneric {

    /**
     * @return Details about the running ASCET. Especially the supported API versions are needed to chose the interface
     *         for further interactions.
     */
    @JsonRequest
    CompletableFuture<AscetDetailsDto> genericGetAscetDetails();

    /**
     * Close the running ASCET-DEVELOPER instance.
     *
     * @return <code>true</code> if the application is going to close,
     *         <code>false</code> if the preconditions are not met, e.g.
     *         dirty editors, running jobs or open dialogs
     */
    @JsonRequest
    CompletableFuture<Boolean> genericCloseAscet();

}
