package com.etas.ascet.api;

import java.util.List;

import org.eclipse.jdt.annotation.NonNullByDefault;

import com.google.auto.value.AutoValue;
import com.ryanharter.auto.value.gson.GenerateTypeAdapter;

/**
 * Notification data
 */
// Checkstyle:off
@NonNullByDefault
@GenerateTypeAdapter
@AutoValue
public abstract class NotificationDto {

    /**
     * About API usage. E.g. the api did not deliver the full information, because a feature is not (yet) supported.
     * <p/>
     * Properties:
     * <ul>
     * <li>{@link #KEY_SEVERITY}</li>
     * <li>{@link #KEY_TEXT}</li>
     * </ul>
     */
    public static final String TOPIC_API = "topic.api";

    /**
     * A console is mirrored here.
     * <p/>
     * Properties:
     * <ul>
     * <li>{@link #KEY_SEVERITY}</li>
     * <li>{@link #KEY_TEXT}</li>
     * </ul>
     */
    public static final String TOPIC_CONSOLE_LOG = "topic.console.log";

    /**
     * The value for this property is a text.
     */
    public static final String KEY_TEXT = "prop.text";

    /**
     * The value for this property is one of:
     * <ul>
     * <li>{@link #SEVERITY_DEBUG}</li>
     * <li>{@link #SEVERITY_INFO}</li>
     * <li>{@link #SEVERITY_WARN}</li>
     * <li>{@link #SEVERITY_ERROR}</li>
     * </ul>
     */
    public static final String KEY_SEVERITY = "prop.severity";

    /**
     * Information that is not intended for the end user, but it might be useful for problem analysis
     */
    public static final String SEVERITY_DEBUG = "DEBUG";
    public static final String SEVERITY_INFO = "INFO";
    public static final String SEVERITY_WARN = "WARN";
    public static final String SEVERITY_ERROR = "ERROR";

    /**
     * Topic of the notification. E.g. {@link #TOPIC_API}.
     */
    public abstract String topic();

    /**
     * The properties of the notification, depending on the {@link #topic()}
     * Properties might be added in future.
     */
    public abstract List<KeyValueDto> properties();

    public static Builder builder() {
        return new AutoValue_NotificationDto.Builder();
    }

    @AutoValue.Builder
    public interface Builder {
        Builder topic(String value);

        Builder properties(List<KeyValueDto> value);

        NotificationDto build();
    }
}
