package com.etas.ascet.api.v1.datatype;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

import com.google.auto.value.AutoValue;
import com.ryanharter.auto.value.gson.GenerateTypeAdapter;

/**
 * Declaration of a global variable or a composite field
 */
// Checkstyle:off
@NonNullByDefault
@GenerateTypeAdapter
@AutoValue
public abstract class DeclarationDtoV1 {

    public abstract String name();

    public abstract VisibilityV1 visibility();

    public abstract boolean isStatic();

    public abstract boolean isProvide();

    public abstract boolean isRequire();

    public abstract DeclKindV1 kind();

    /**
     * The type of the variable or field
     */
    public abstract IdentifierDtoV1 type();

    @Nullable
    public abstract ScalarRepresentationDtoV1 scalarRepresentation();

    public static Builder builder() {
        return new AutoValue_DeclarationDtoV1.Builder();
    }

    @AutoValue.Builder
    public interface Builder {

        /**
         * @see DeclarationDtoV1#name()
         */
        Builder name(String value);

        /**
         * @see DeclarationDtoV1#type()
         */
        Builder type(IdentifierDtoV1 value);

        /**
         * @see DeclarationDtoV1#visibility()
         */
        Builder visibility(VisibilityV1 value);

        /**
         * @see DeclarationDtoV1#isStatic()
         */
        Builder isStatic(boolean value);

        /**
         * @see DeclarationDtoV1#isProvide()
         */
        Builder isProvide(boolean value);

        /**
         * @see DeclarationDtoV1#isRequire()
         */
        Builder isRequire(boolean value);

        /**
         * @see DeclarationDtoV1#kind()
         */
        Builder kind(DeclKindV1 value);

        /**
         * @see DeclarationDtoV1#scalarRepresentation()
         */
        Builder scalarRepresentation(@Nullable ScalarRepresentationDtoV1 value);

        DeclarationDtoV1 build();
    }
}
