package com.etas.ascet.api.v1.datatype;

import java.util.List;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

import com.etas.ascet.api.KeyValueDto;
import com.google.auto.value.AutoValue;
import com.ryanharter.auto.value.gson.GenerateTypeAdapter;

/**
 * Specifies the dimension of dimensional {@link TargetInstanceDtoV1} like arrays/matrices or curves/maps.
 */
// Checkstyle:off
@NonNullByDefault
@GenerateTypeAdapter
@AutoValue
public abstract class TargetInstanceDimensionDtoV1 {

    /**
     * The actual size of array or table. (Table might be expandable towards maxSize)
     */
    public abstract int size();

    /**
     * In case of an array dimension bound defined with a system constant: this names that system constant.
     * For all other cases this is an empty string.
     * This name is the generated symbol (vs. model qualified name).
     */
    public abstract String sizeSymbol();

    /**
     * Specific for table types, which might be expandable to the maxSize
     */
    @Nullable
    public abstract Integer maxSize();

    /**
     * Normally this is 0 .. size.<br/>
     * But in case the first index is not zero, then the indexRange can be shifted.
     */
    @Nullable
    public abstract NumberRangeDtoV1 indexRange();

    public abstract IdentifierDtoV1 indexType();

    /**
     * The value of {@code @a2l_unit_label} annotation
     */
    public abstract String a2lUnit();

    public abstract ScalarRepresentationDtoV1 indexRepresentation();

    @Nullable
    public abstract IdentifierDtoV1 axisIdentifier();

    public static final String PROPERTY_KEY_ASAP2_COEFFICIENTS =
            TargetInstanceDtoV1.PROPERTY_KEY_ASAP2_COEFFICIENTS;

    public abstract List<KeyValueDto> properties();

    public static Builder builder() {
        return new AutoValue_TargetInstanceDimensionDtoV1.Builder();
    }

    @AutoValue.Builder
    public interface Builder {
        /**
         * @see TargetInstanceDimensionDtoV1#size()
         */
        Builder size(int value);

        /**
         * @see TargetInstanceDimensionDtoV1#sizeSymbol()
         */
        Builder sizeSymbol(String value);

        /**
         * @see TargetInstanceDimensionDtoV1#maxSize()
         */
        Builder maxSize(@Nullable Integer value);

        /**
         * @see TargetInstanceDimensionDtoV1#indexRange()
         */
        Builder indexRange(@Nullable NumberRangeDtoV1 value);

        /**
         * @see TargetInstanceDimensionDtoV1#indexType()
         */
        Builder indexType(IdentifierDtoV1 value);

        /**
         * @see TargetInstanceDimensionDtoV1#a2lUnit()
         */
        Builder a2lUnit(String value);

        /**
         * @see TargetInstanceDimensionDtoV1#indexRepresentation()
         */
        Builder indexRepresentation(ScalarRepresentationDtoV1 value);

        /**
         * @see TargetInstanceDimensionDtoV1#axisIdentifier()
         */
        Builder axisIdentifier(@Nullable IdentifierDtoV1 value);

        /**
         * @see TargetInstanceDimensionDtoV1#properties()
         */
        Builder properties(List<KeyValueDto> value);

        TargetInstanceDimensionDtoV1 build();
    }
}
