package com.etas.ascet.api.v1.datatype;

import org.eclipse.jdt.annotation.NonNullByDefault;

import com.google.auto.value.AutoValue;
import com.ryanharter.auto.value.gson.GenerateTypeAdapter;

// Checkstyle:off
@NonNullByDefault
@GenerateTypeAdapter
@AutoValue
/**
 * Only applicable for instances, that cannot contain child instances.
 */
public abstract class TargetInstanceValueDtoV1 {

    /**
     * The value-type of the object. <br/>
     * Example 1: For scalars, this is the same as {@linkplain #type()}.<br/>
     * Example 2: In an array/matrix/table, it is the type of the element as result of indexed/interpolated access.
     */
    public abstract IdentifierDtoV1 type();

    /**
     * representation of the value in the generated code.
     */
    public abstract ScalarRepresentationDtoV1 representation();

    public abstract boolean isDeltaTElement();

    /**
     * initializer value as nested arrays of numbers
     */
    public abstract NumberInitializerDtoV1 initialValue();

    public static Builder builder() {
        return new AutoValue_TargetInstanceValueDtoV1.Builder();
    }

    @AutoValue.Builder
    public interface Builder {
        /**
         * @see TargetInstanceValueDtoV1#type()
         */
        Builder type(IdentifierDtoV1 value);

        /**
         * @see TargetInstanceValueDtoV1#representation()
         */
        Builder representation(ScalarRepresentationDtoV1 value);

        /**
         * @see TargetInstanceValueDtoV1#isDeltaTElement()
         */
        Builder isDeltaTElement(boolean value);

        /**
         * @see TargetInstanceValueDtoV1#initialValue()
         */
        Builder initialValue(NumberInitializerDtoV1 value);

        TargetInstanceValueDtoV1 build();
    }
}
