package com.etas.ascet.api.v1.datatype;

import java.util.List;

import org.eclipse.jdt.annotation.NonNullByDefault;

import com.google.auto.value.AutoValue;
import com.ryanharter.auto.value.gson.GenerateTypeAdapter;

// Checkstyle:off
/**
 * Represents methods as they are generated for the target (vs. model method).
 * Thus, especially the representation of the method is being considered here and
 * for the same model method, there might be multiple {@link TargetmethodDtoV1}.
 */
@NonNullByDefault
@GenerateTypeAdapter
@AutoValue
public abstract class TargetMethodDtoV1 {

    public abstract IdentifierDtoV1 identifier();

    /**
     * Identifier for the surrounding model class
     */
    public abstract IdentifierDtoV1 containerClass();

    /**
     * Annotations attached to the method
     */
    public abstract List<AnnotationDtoV1> annotations();

    /**
     * Kind of method, e.g. method or initializer
     */
    public abstract MethodKindV1 methodKind();

    /**
     * In ESDL those are public methods with zero args, void return type and a {@code @}thread annotation
     */
    public abstract boolean isProcessMethod();

    /**
     * The generated method name.
     */
    public abstract String generatedName();

    public static Builder builder() {
        return new AutoValue_TargetMethodDtoV1.Builder()
                .isProcessMethod(false);
    }

    @AutoValue.Builder
    public interface Builder {

        /**
         * @see TargetMethodDtoV1#identifier()
         */
        Builder identifier(IdentifierDtoV1 value);

        /**
         * @see TargetMethodDtoV1#containerClass()
         */
        Builder containerClass(IdentifierDtoV1 value);

        /**
         * @see TargetMethodDtoV1#annotations()
         */
        Builder annotations(List<AnnotationDtoV1> value);

        /**
         * @see TargetMethodDtoV1#methodKind()
         */
        Builder methodKind(MethodKindV1 value);

        /**
         * @see TargetMethodDtoV1#isProcessMethod()
         */
        Builder isProcessMethod(boolean value);

        /**
         * @see TargetMethodDtoV1#generatedName()
         */
        Builder generatedName(String value);

        TargetMethodDtoV1 build();

    }
}
